@@ -1,6 +1,6 @@
-/*                                                                                                                            
+/*
 
-Copyright (C) 2008-2015 Michele Martone
+Copyright (C) 2008-2021 Michele Martone
 
 This file is part of librsb.
 
@@ -51,7 +51,7 @@ If not, see <http://www.gnu.org/licenses/>.
 #include <sys/stat.h>
 #endif
 
-#define rsb_util_sort_column_major(VA,IA,JA,nnz,nr,nc,typecode,flags) rsb_util_sort_row_major_inner(VA,JA,IA,nnz,nc,nr,typecode,flags)
+#define rsb_util_sort_column_major(VA,IA,JA,nnz,nr,nc,typecode,flags) rsb__util_sort_row_major_inner(VA,JA,IA,nnz,nc,nr,typecode,flags)
 #if 1
 #define RSB_IO_VERBOSE_MSG(NZI,NNZ)
 #else
@@ -62,8 +62,19 @@ If not, see <http://www.gnu.org/licenses/>.
 
 #define RSB_FILE_ALLOW_LOAD_EMPTY_PATTERN 1 /* 20140324 */
 
+RSB_INTERNALS_COMMON_HEAD_DECLS
+
+#if RSB_WANT_DEBUG_VERBOSE_INTERFACE_NOTICE
+#define RSB__VERBOSE_IO_ERRORS rsb_global_session_handle.rsb_g_verbose_interface
+#define RSB__VERBOSE_IO_WARNINGS rsb_global_session_handle.rsb_g_verbose_interface ? RSB_BOOL_TRUE : RSB_BOOL_FALSE
+#else /* RSB_WANT_DEBUG_VERBOSE_INTERFACE_NOTICE */
+#define RSB__VERBOSE_IO_ERRORS 0
+#define RSB__VERBOSE_IO_WARNINGS RSB_BOOL_FALSE
+#endif /* RSB_WANT_DEBUG_VERBOSE_INTERFACE_NOTICE */
+
 #ifdef RSB_WITH_MM
-rsb_err_t rsb_util_mm_load_coo_matrix(const char *filename, struct rsb_coo_matrix_t * cmp)
+#ifdef RSB_OBSOLETE_QUARANTINE_UNUSED
+rsb_err_t rsb__util_mm_load_coo_matrix(const char *filename, struct rsb_coo_mtx_t * cmp)
 {
 	/**
 	 * \ingroup gr_internals
@@ -75,7 +86,7 @@ rsb_err_t rsb_util_mm_load_coo_matrix(const char *filename, struct rsb_coo_matri
 	 * \note used by experiment.c files
 	 */
 	rsb_err_t errval = RSB_ERR_NO_ERROR;
-	struct rsb_coo_matrix_t cm;
+	struct rsb_coo_mtx_t cm;
 	rsb_flags_t flags = RSB_FLAG_NOFLAGS;
 
 	if(!cmp || !filename)
@@ -89,11 +100,12 @@ rsb_err_t rsb_util_mm_load_coo_matrix(const char *filename, struct rsb_coo_matri
 	errval = rsb__util_mm_load_matrix_f(filename, &cm.IA, &cm.JA,&cm.VA , &cm.nr, &cm.nc, &cm.nnz , cm.typecode, flags, NULL, NULL);
 	if(RSB_SOME_ERROR(errval))
 		goto err;
-	rsb_memcpy(cmp,&cm,sizeof(cm));
+	rsb__memcpy(cmp,&cm,sizeof(cm));
 err:
 	rsb__do_perror(NULL,errval);
 	RSB_DO_ERR_RETURN(errval)
 }
+#endif /* RSB_OBSOLETE_QUARANTINE_UNUSED */
 
 rsb_err_t rsb__util_mm_info_matrix_f(const char *fn,  rsb_coo_idx_t *m, rsb_coo_idx_t *k , rsb_nnz_idx_t *nnz, rsb_type_t *typecode, rsb_bool_t * is_symmetric, rsb_bool_t * is_hermitian, rsb_bool_t * is_pattern, rsb_bool_t * is_lower, rsb_bool_t * is_upper , rsb_bool_t * is_vector)
 {
@@ -103,10 +115,11 @@ rsb_err_t rsb__util_mm_info_matrix_f(const char *fn,  rsb_coo_idx_t *m, rsb_coo_
 	 * */
 
   	FILE * fd = NULL;
-	int innz = 0;	/* FIXME */
-	int _m = 0,_k = 0;
+	rsb_coo_idx_t innz = 0;	/* FIXME */
+	rsb_coo_idx_t _m = 0,_k = 0;
 	char matcode[RSB_MMIOH_CL]; // !?
 	rsb_bool_t is_vector_ = RSB_BOOL_FALSE;
+	int rbc = 0;
 
 	if(nnz)*nnz = RSB_MARKER_NNZ_VALUE ;
 	if(m)*m = RSB_MARKER_COO_VALUE;
@@ -122,13 +135,21 @@ rsb_err_t rsb__util_mm_info_matrix_f(const char *fn,  rsb_coo_idx_t *m, rsb_coo_
 	if(fd==NULL)
 	if ((fd = RSB_FOPEN(fn, "r")) == NULL)
 	{
-		RSB_STDERR("Failed opening file: %s\n",fn);
+		if( RSB__VERBOSE_IO_ERRORS )
+			RSB_STDERR("Failed opening file: %s\n",fn);
 		return RSB_ERR_GENERIC_ERROR;
 	}
 
-	if (rsb__mm_read_banner(fd,NULL,&(matcode)) != 0)
+#if RSB_WANT_ZLIB_SUPPORT
+	if ((rbc=rsb__mm_read_banner(NULL,fd,&(matcode))) != 0)
+#else
+	if ((rbc=rsb__mm_read_banner(fd,NULL,&(matcode))) != 0)
+#endif
 	{
-        	RSB_STDERR("Could not process Matrix Market banner.\n");
+		if( rbc == MM_LIKELY_GZIPPED_FILE )
+        		RSB_STDERR("Trying to load a gzipped file as matrix without gzip decoder in ?!\n");
+		if( RSB__VERBOSE_IO_ERRORS )
+        		RSB_STDERR("Could not process Matrix Market banner.\n");
 		RSB_FCLOSE(fd);
 		return RSB_ERR_GENERIC_ERROR;
 	}
@@ -136,20 +157,26 @@ rsb_err_t rsb__util_mm_info_matrix_f(const char *fn,  rsb_coo_idx_t *m, rsb_coo_
 	/*  This is how one can screen matrix types if their application */
 	/*  only supports a subset of the Matrix Market data types.      */
 
-	is_vector_ = (rsb_mm_is_sparse(matcode))?RSB_BOOL_FALSE:RSB_BOOL_TRUE;
-	if ( !rsb_mm_is_matrix(matcode) /*|| !rsb_mm_is_sparse(matcode)*/ )
-//	if (!rsb_mm_is_real(matcode) || !rsb_mm_is_matrix(matcode) || !rsb_mm_is_sparse(matcode) )
+	if ( !rsb_mm_is_matrix(matcode) )
 	{
-        	RSB_STDERR("%s","Sorry, this application does not support ");
-	        RSB_STDERR("Matrix Market type: [%s]\n", rsb__mm_typecode_to_str(matcode));
+    		char *mstr = rsb__mm_typecode_to_str(matcode);
+		RSB_STDERR("Sorry, this application does not support "
+				"Matrix Market type: [%s]\n", mstr?mstr:"?");
 		RSB_FCLOSE(fd);
-        	return RSB_ERR_UNSUPPORTED_TYPE;
+	       	if(mstr)
+			free(mstr);
+ 		return RSB_ERR_UNSUPPORTED_TYPE;
 	}
 
+	is_vector_ = (rsb_mm_is_sparse(matcode))?RSB_BOOL_FALSE:RSB_BOOL_TRUE;
+
 	/* find out size of sparse matrix .... */
 
-		
+#if RSB_WANT_ZLIB_SUPPORT
+	if( ((is_vector_) && (rsb__mm_read_mtx_array_size(NULL,fd,&_m,&_k) !=0)) || ((!is_vector_) && (rsb__mm_read_mtx_crd_size(NULL,fd,&_m,&_k,&innz)) !=0) )
+#else
 	if( ((is_vector_) && (rsb__mm_read_mtx_array_size(fd,NULL,&_m,&_k) !=0)) || ((!is_vector_) && (rsb__mm_read_mtx_crd_size(fd,NULL,&_m,&_k,&innz)) !=0) )
+#endif
 	{
 		RSB_FCLOSE(fd);
         	return RSB_ERR_GENERIC_ERROR;
@@ -188,16 +215,13 @@ rsb_err_t rsb__util_mm_info_matrix_f(const char *fn,  rsb_coo_idx_t *m, rsb_coo_
 
 	if(is_lower)
 	{
-		// FIXME: using the default. UNFINISHED
-		if(is_symmetric || is_hermitian)
-			*is_lower |= RSB_BOOL_TRUE;
-		else
-			*is_lower |= RSB_BOOL_FALSE;
+		// TODO
 	}
 
 	if(is_upper)
-		*is_upper |= RSB_BOOL_FALSE; // FIXME: why this ?
-		//*is_upper |= RSB_BOOL_TRUE;
+	{
+		// TODO
+	}
 
 	if(m && k)
 	if (((int)*m != _m)||((int)*k != _k))
@@ -228,34 +252,37 @@ static int rsb_zfscanf(FILE * fd,const char * fs,rsb_coo_idx_t *IV, rsb_coo_idx_
 {
 	/**
 	 *  \ingroup internals
-	 *  FIXME: error handling is missing
 	 * */
-#if RSB_WANT_ZLIB_SUPPORT
-	if(ngzfd)
+	if(fd)
 	{
 		if((!IV) && (!JV))
 		{
 			if(VAI)
-				return fscanf(ngzfd,fs,VAR,VAI);
+				return fscanf(fd,fs,VAR,VAI);
 			else
-				return fscanf(ngzfd,fs,VAR);
+				return fscanf(fd,fs,VAR);
 		}
 		else
 		{
 			if(VAI)
-				return fscanf(ngzfd,fs,IV,JV,VAR,VAI);
+				return fscanf(fd,fs,IV,JV,VAR,VAI);
 			if(VAR)
-				return fscanf(ngzfd,fs,IV,JV,VAR);
+				return fscanf(fd,fs,IV,JV,VAR);
 			else
-				return fscanf(ngzfd,fs,IV,JV);
+				return fscanf(fd,fs,IV,JV);
 		}
 	}
 	else
+	{
+#if RSB_WANT_ZLIB_SUPPORT
+		return rsb__fscanf(ngzfd,fs,IV,JV,VAR,VAI);
+#else /* RSB_WANT_ZLIB_SUPPORT */
+		return 0; /* error case */
 #endif /* RSB_WANT_ZLIB_SUPPORT */
-		return rsb_fscanf(fd,fs,IV,JV,VAR,VAI);
+	}
 }
 
-int rsb_fscanf(FILE * fd,const char * fs,rsb_coo_idx_t *IV, rsb_coo_idx_t *JV, void * VAR, void * VAI)
+int rsb__fscanf(FILE * fd,const char * fs,rsb_coo_idx_t *IV, rsb_coo_idx_t *JV, void * VAR, void * VAI)
 {
 	/**
 	 *  \ingroup internals
@@ -263,7 +290,7 @@ int rsb_fscanf(FILE * fd,const char * fs,rsb_coo_idx_t *IV, rsb_coo_idx_t *JV, v
 	 * */
 #if RSB_WANT_ZLIB_SUPPORT
 	char line[MM_MAX_LINE_LENGTH];
-	gzgets(fd,line,MM_MAX_LINE_LENGTH);
+	gzgets((gzFile)fd,line,MM_MAX_LINE_LENGTH);
 
 	if((!IV) && (!JV))
 	{
@@ -305,13 +332,13 @@ int rsb_fscanf(FILE * fd,const char * fs,rsb_coo_idx_t *IV, rsb_coo_idx_t *JV, v
 #endif /* RSB_WANT_ZLIB_SUPPORT */
 }
 
-char * rsb_fgets(char* RSB_RESTRICT buf, int len, FILE * RSB_RESTRICT fd)
+char * rsb__fgets(char* RSB_RESTRICT buf, int len, FILE * RSB_RESTRICT fd)
 {
 	/**
 	 *  \ingroup internals
 	 * */
 #if RSB_WANT_ZLIB_SUPPORT
-	return gzgets(fd,buf,len);
+	return gzgets((gzFile)fd,buf,len);
 #else /* RSB_WANT_ZLIB_SUPPORT */
 	return fgets(buf,len,fd);
 #endif /* RSB_WANT_ZLIB_SUPPORT */
@@ -325,6 +352,97 @@ rsb_err_t rsb__util_mm_load_vector_f(const char *fn, void **VA, rsb_nnz_idx_t *n
 	return errval;
 }
 
+#if RSB_WANT_EXPERIMENTAL_BINARY_COO
+int rsb_getc(FILE *fd, void *ngzfd)
+{
+	/**
+	 * \ingroup gr_internals
+	 * */
+	return fd ? getc(fd) : RSB_GETC(ngzfd);
+}
+
+int rsb_ungetc(int cc, FILE *fd, void *ngzfd)
+{
+	/**
+	 * \ingroup gr_internals
+	 * */
+	return fd ? ungetc(cc,fd) : RSB_UNGETC(cc,ngzfd);
+}
+
+size_t rsb_fread(void* buf, size_t size, size_t nitems, void * fd, void *ngzfd)
+{
+	/**
+	 * \ingroup gr_internals
+	 * */
+	return fd ?
+		fread(buf, size, nitems, fd)
+		:
+		RSB_FREAD(buf, size, nitems, ngzfd);
+}
+
+rsb_err_t rsb__read_coo_bin_fd(FILE *fd, void *ngzfd, rsb_coo_idx_t * IA, rsb_coo_idx_t * JA, void *VA, const rsb_coo_idx_t nr, const rsb_coo_idx_t nc, const rsb_nnz_idx_t nnz, const rsb_type_t typecode)
+{
+	/**
+	 * \ingroup gr_internals
+	 * */
+	rsb_err_t errval = RSB_ERR_NO_ERROR;
+	int c = rsb_getc(fd, ngzfd);
+	int l = c;
+	const size_t nmemb = nnz;
+	void * TA = NULL;
+	rsb_nnz_idx_t i;
+
+	while( ( c = rsb_getc(fd, ngzfd) ) )
+		l = c;
+	if( RSB_MATRIX_UNSUPPORTED_TYPE(l) )
+	{
+		errval = RSB_ERR_UNSUPPORTED_TYPE;
+		RSB_PERR_GOTO(err,RSB_ERRM_UNSUPPORTED_TYPE);
+	}
+	if( l != typecode )
+	{
+		TA = rsb__malloc_vector(nmemb,l);
+		if ( !TA )
+		{
+			errval = RSB_ERR_ENOMEM;
+			RSB_PERR_GOTO(err,RSB_ERRM_ENOMEM);
+		}
+	}
+	else
+		TA = VA;
+	errval = RSB_ERR_INTERNAL_ERROR;
+	if( rsb_fread( IA,sizeof(rsb_coo_idx_t),nmemb,fd,ngzfd) != nmemb )
+		RSB_PERR_GOTO(err,RSB_ERRM_ES);
+	if( rsb_fread( JA,sizeof(rsb_coo_idx_t),nmemb,fd,ngzfd) != nmemb )
+		RSB_PERR_GOTO(err,RSB_ERRM_ES);
+	if( rsb_fread( TA,RSB_SIZEOF(l),nmemb,fd,ngzfd) != nmemb )
+		RSB_PERR_GOTO(err,RSB_ERRM_ES);
+	errval = RSB_ERR_NO_ERROR;
+	for ( i=0; i < nnz; i++ )
+	{
+		(IA)[i]--, (JA)[i]--; /* adjust from 1-based to 0-based (may revive rsb__util_nnz_array_from_fortran_indices for this ?) */
+		if ( IA[i] < 0 || IA[i] > nr || JA[i] < 0 || JA[i] > nc )
+		{
+			// TODO: move this check to be called from rsb__util_mm_load_matrix_f()
+			errval = RSB_ERR_CORRUPT_INPUT_DATA;
+			RSB_PERR_GOTO(err,RSB_ERRM_CORRUPT_INPUT_DATA);
+		}
+	}
+
+	if( l != typecode )
+		errval = rsb__do_copy_converted_scaled(TA, VA, NULL, l, typecode, nnz, RSB_TRANSPOSITION_N);
+err:
+	if( l != typecode )
+		RSB_CONDITIONAL_FREE(TA);
+	return errval;
+}
+#endif /* RSB_WANT_EXPERIMENTAL_BINARY_COO */
+
+#ifdef RSB_WANT_LONG_IDX_TYPE 
+#define RSB_IJPC "%lld %lld"
+#else /* RSB_WANT_LONG_IDX_TYPE */
+#define RSB_IJPC   "%d %d"
+#endif /* RSB_WANT_LONG_IDX_TYPE */
 rsb_err_t rsb__util_mm_load_matrix_f(const char *fn, rsb_coo_idx_t ** IA, rsb_coo_idx_t ** JA, void **VA, rsb_coo_idx_t *m, rsb_coo_idx_t *k , rsb_nnz_idx_t *nnz, rsb_type_t typecode, rsb_flags_t flags, rsb_bool_t *is_lowerp, rsb_bool_t *is_upperp)
 {
 	/**
@@ -362,6 +480,7 @@ rsb_err_t rsb__util_mm_load_matrix_f(const char *fn, rsb_coo_idx_t ** IA, rsb_co
 //#if RSB_WANT_ZLIB_SUPPORT
 	FILE *ngzfd = NULL;
 //#endif
+	const rsb_bool_t wv = RSB__VERBOSE_IO_WARNINGS;
 	rsb_nnz_idx_t i = 0;
 	rsb_nnz_idx_t annz = 0;/* allocated nnz */
 	rsb_bool_t is_symmetric = RSB_BOOL_FALSE,is_lower = RSB_BOOL_FALSE,is_upper = RSB_BOOL_FALSE,is_pattern = RSB_BOOL_FALSE, is_hermitian = RSB_BOOL_FALSE, is_vector = RSB_BOOL_FALSE;/* FIXME : no expansion support for hermitian */
@@ -369,7 +488,9 @@ rsb_err_t rsb__util_mm_load_matrix_f(const char *fn, rsb_coo_idx_t ** IA, rsb_co
 	rsb_flags_t otype = typecode;/* original type */
 	rsb_time_t frt = 0;
 	char matcode[RSB_MMIOH_CL]; // !?
+#if RSB_WANT_ZLIB_SUPPORT
 	rsb_bool_t is_gz = RSB_BOOL_FALSE;
+#endif /* RSB_WANT_ZLIB_SUPPORT */
 
 	#ifdef RSB_NUMERICAL_TYPE_DOUBLE
 	double  **dval = NULL;
@@ -389,8 +510,8 @@ rsb_err_t rsb__util_mm_load_matrix_f(const char *fn, rsb_coo_idx_t ** IA, rsb_co
 	#ifdef RSB_NUMERICAL_TYPE_DOUBLE_COMPLEX
 	double complex  **Zval = NULL;
 	#endif /* RSB_NUMERICAL_TYPE_DOUBLE_COMPLEX */
-	int innz = 0;	/* FIXME */
-	int _m = 0,_k = 0;
+	rsb_coo_idx_t innz = 0;
+	rsb_coo_idx_t _m = 0,_k = 0;
 #if RSB_20120321_IOBUFFERING
 	char*iobuf = NULL;
 	size_t iobbs = 16*1024*1024;
@@ -466,42 +587,44 @@ rsb_err_t rsb__util_mm_load_matrix_f(const char *fn, rsb_coo_idx_t ** IA, rsb_co
   	
 	{
 		// THIS IS A CODE DUPLICATION ..
-	is_gz = RSB_BOOL_FALSE;
 #if RSB_WANT_ZLIB_SUPPORT
-	if ((fd = gzopen(fn,"r")) != NULL)
+	is_gz = RSB_BOOL_FALSE;
+	if ((fd = (FILE*)gzopen(fn,"r")) != NULL)
 	{
-		if(gzdirect(fd))
-		{
-			gzclose(fd);
-		}
+		if(gzdirect((gzFile)fd))
+			;
 		else
 			is_gz = RSB_BOOL_TRUE;
+		RSB_FCLOSE(fd);
+		fd=NULL;
 	}
 #endif /* RSB_WANT_ZLIB_SUPPORT */
 
 #if RSB_WANT_ZLIB_SUPPORT
 	if(is_gz)
 	{
-		if((fd = gzopen(fn,"r")) == NULL)
+		if((fd = (FILE*)gzopen(fn,"r")) == NULL)
 		{
 			/* TODO: the following code is not robust, shall fix it. */
 #ifndef RSB_HAVE_DUP
 #error Functions 'dup/fileno' is not present ? Reconfigure without Z library then!
-			int fnum = 0;/* */
+			const int fnum = 0;/* */
 			ngzfd = NULL;
 #else /* RSB_HAVE_DUP */
-			int fnum = dup( rsb__fileno(fd) );
-			ngzfd = gzdopen(fnum,"r");
+			const int fnum = dup( rsb__fileno(fd) );
+			ngzfd = (FILE*)gzdopen(fnum,"r");
 #endif /* RSB_HAVE_DUP */
 			if(!ngzfd)
 			{
-				gzclose(fd);
+				RSB_FCLOSE(fd);
 				RSB_ERROR(RSB_ERRMSG_FILEOPENPGZ"\n");
 				return RSB_ERR_GENERIC_ERROR;
 			}
 			else
 				;
 		}
+		else
+			ngzfd = fd, fd = NULL;
 	}
 	else
 #endif /* RSB_WANT_ZLIB_SUPPORT */
@@ -522,7 +645,7 @@ rsb_err_t rsb__util_mm_load_matrix_f(const char *fn, rsb_coo_idx_t ** IA, rsb_co
 		return RSB_ERR_GENERIC_ERROR;
 	}
 	else
-		ngzfd = fd;
+		;//ngzfd = fd;
 #if RSB_20120321_IOBUFFERING
 	//iobbs = BUFSIZ;
 	if(iobbs>0)
@@ -684,10 +807,21 @@ rsb_err_t rsb__util_mm_load_matrix_f(const char *fn, rsb_coo_idx_t ** IA, rsb_co
 	if (typecode == RSB_NUMERICAL_TYPE_INT)
 	for (i=0; i<*nnz; i++)
 	{
-		double fv;
-		re += (rsb_zfscanf(fd,"%lg\n",NULL,NULL,&fv,NULL,ngzfd)==1);
-		(*ival)[i] = (int)fv;
-		RSB_IO_VERBOSE_MSG(i,*nnz);
+		double fv,iv;
+		if(rsb_mm_is_complex(matcode))
+		for (i=0; i<*nnz; i++)
+		{
+			re += (rsb_zfscanf(fd,"%lg %lg\n",NULL,NULL,&(fv),&(iv),ngzfd)==2);
+			(*ival)[i] = (int)fv;
+			RSB_IO_VERBOSE_MSG(i,*nnz);
+		}
+		else
+		for (i=0; i<*nnz; i++)
+		{
+			re += (rsb_zfscanf(fd,"%lg\n",NULL,NULL,&fv,NULL,ngzfd)==1);
+			(*ival)[i] = (int)fv;
+			RSB_IO_VERBOSE_MSG(i,*nnz);
+		}
 	}
 	#endif /* RSB_NUMERICAL_TYPE_INT */
 
@@ -747,6 +881,29 @@ rsb_err_t rsb__util_mm_load_matrix_f(const char *fn, rsb_coo_idx_t ** IA, rsb_co
 	#endif /* RSB_NUMERICAL_TYPE_DOUBLE_COMPLEX */
 	goto scan_done;
 full_scan:
+
+#if RSB_WANT_EXPERIMENTAL_BINARY_COO
+	if( IA && JA && VA && *IA && *JA && *VA && (fd || ngzfd) )
+	{
+		const int cc = rsb_getc(fd, ngzfd);
+
+		if(cc == 'B') // Binary..
+		{
+			if(RSB_SOME_ERROR(errval = rsb__read_coo_bin_fd(fd, ngzfd, *IA, *JA, *VA, *m, *k , *nnz, otype)))
+			{
+				RSB_PERR_GOTO(err,RSB_ERRM_ES);
+			}
+			else
+			{
+				re = *nnz;
+				goto scan_done;
+			}
+		}
+		else
+			rsb_ungetc(cc,fd,ngzfd);
+	}
+#endif /* RSB_WANT_EXPERIMENTAL_BINARY_COO */
+
 	/* NOTE: when reading in doubles, ANSI C requires the use of the "l"  */
 	/*   specifier as in "%lg", "%lf", "%le", otherwise errors will occur */
 	/*  (ANSI C X3.159-1989, Sec. 4.9.6.2, p. 136 lines 13-15)            */
@@ -760,11 +917,11 @@ full_scan:
 #else /* RSB_WANT_OMPIO_SUPPORT */
 		for (i=0; i<*nnz; i++)
 		{
-			int iI,iJ;
+			rsb_coo_idx_t iI,iJ;
 			double iv;
-			re += (rsb_zfscanf(fd,"%d %d %lg %lg\n",&iI,&iJ,*dval+i,&(iv),ngzfd)==4);
-			(*IA)[i] = (rsb_coo_idx_t)iI;
-			(*JA)[i] = (rsb_coo_idx_t)iJ;
+			re += (rsb_zfscanf(fd,RSB_IJPC " %lg %lg\n",&iI,&iJ,*dval+i,&(iv),ngzfd)==4);
+			(*IA)[i] = iI;
+			(*JA)[i] = iJ;
 	        	(*IA)[i]--;  /* adjust from 1-based to 0-based */
 	        	(*JA)[i]--;
 			RSB_IO_VERBOSE_MSG(i,*nnz);
@@ -776,10 +933,10 @@ full_scan:
 #else /* RSB_WANT_OMPIO_SUPPORT */
 		for (i=0; i<*nnz; i++)
 		{
-			int iI,iJ;
-			re += (rsb_zfscanf(fd,"%d %d %lg\n",&iI,&iJ,*dval+i,NULL,ngzfd)==3);
-			(*IA)[i] = (rsb_coo_idx_t)iI;
-			(*JA)[i] = (rsb_coo_idx_t)iJ;
+			rsb_coo_idx_t iI,iJ;
+			re += (rsb_zfscanf(fd,RSB_IJPC " %lg\n",&iI,&iJ,*dval+i,NULL,ngzfd)==3);
+			(*IA)[i] = iI;
+			(*JA)[i] = iJ;
 	        	(*IA)[i]--;  /* adjust from 1-based to 0-based */
         		(*JA)[i]--;
 			RSB_IO_VERBOSE_MSG(i,*nnz);
@@ -797,11 +954,11 @@ full_scan:
 #else /* RSB_WANT_OMPIO_SUPPORT */
 		for (i=0; i<*nnz; i++)
 		{
-			int iI,iJ;
+			rsb_coo_idx_t iI,iJ;
 			float iv;
-			re += (rsb_zfscanf(fd,"%d %d %g %g\n",&iI,&iJ,*fval+i,&(iv),ngzfd)==4);
-			(*IA)[i] = (rsb_coo_idx_t)iI;
-			(*JA)[i] = (rsb_coo_idx_t)iJ;
+			re += (rsb_zfscanf(fd,RSB_IJPC " %g %g\n",&iI,&iJ,*fval+i,&(iv),ngzfd)==4);
+			(*IA)[i] = iI;
+			(*JA)[i] = iJ;
 	        	(*IA)[i]--;  /* adjust from 1-based to 0-based */
 	        	(*JA)[i]--;
 			RSB_IO_VERBOSE_MSG(i,*nnz);
@@ -813,10 +970,10 @@ full_scan:
 #else /* RSB_WANT_OMPIO_SUPPORT */
 		for (i=0; i<*nnz; i++)
 		{
-			int iI,iJ;
-			re += (rsb_zfscanf(fd, "%d %d %g\n",&iI,&iJ,*fval+i,NULL,ngzfd)==3);
-			(*IA)[i] = (rsb_coo_idx_t)iI;
-			(*JA)[i] = (rsb_coo_idx_t)iJ;
+			rsb_coo_idx_t iI,iJ;
+			re += (rsb_zfscanf(fd,RSB_IJPC " %g\n",&iI,&iJ,*fval+i,NULL,ngzfd)==3);
+			(*IA)[i] = iI;
+			(*JA)[i] = iJ;
 	        	(*IA)[i]--;  /* adjust from 1-based to 0-based */
 	        	(*JA)[i]--;
 			RSB_IO_VERBOSE_MSG(i,*nnz);
@@ -827,22 +984,42 @@ full_scan:
 
 	#ifdef RSB_NUMERICAL_TYPE_INT
 	if (typecode == RSB_NUMERICAL_TYPE_INT)
+	{
 #if RSB_WANT_OMPIO_SUPPORT
 			rsb_ompio_INT(nnz,fd,ngzfd,dval,IA,JA,&re);
+#else /* RSB_WANT_OMPIO_SUPPORT */
+	if(rsb_mm_is_complex(matcode))
+#if RSB_WANT_OMPIO_SUPPORT
+	{errval = RSB_ERR_UNIMPLEMENTED_YET;RSB_PERR_GOTO(err,RSB_ERRM_ES);}
 #else /* RSB_WANT_OMPIO_SUPPORT */
 	for (i=0; i<*nnz; i++)
 	{
-		int iI,iJ;
-		double fv;
-		re += (rsb_zfscanf(fd,"%d %d %lg\n",&iI,&iJ,&fv,NULL,ngzfd)==3);
-		(*IA)[i] = (rsb_coo_idx_t)iI;
-		(*JA)[i] = (rsb_coo_idx_t)iJ;
+		rsb_coo_idx_t iI,iJ;
+		double fv,iv;
+		re += (rsb_zfscanf(fd,RSB_IJPC " %lg %lg\n",&iI,&iJ,&fv,&iv,ngzfd)==4);
+		(*IA)[i] = iI;
+		(*JA)[i] = iJ;
         	(*IA)[i]--;  /* adjust from 1-based to 0-based */
         	(*JA)[i]--;
 		(*ival)[i] = (int)fv;
 		RSB_IO_VERBOSE_MSG(i,*nnz);
 	}
 #endif /* RSB_WANT_OMPIO_SUPPORT */
+	else
+	for (i=0; i<*nnz; i++)
+	{
+		rsb_coo_idx_t iI,iJ;
+		double fv;
+		re += (rsb_zfscanf(fd,RSB_IJPC " %lg\n",&iI,&iJ,&fv,NULL,ngzfd)==3);
+		(*IA)[i] = iI;
+		(*JA)[i] = iJ;
+        	(*IA)[i]--;  /* adjust from 1-based to 0-based */
+        	(*JA)[i]--;
+		(*ival)[i] = (int)fv;
+		RSB_IO_VERBOSE_MSG(i,*nnz);
+	}
+#endif /* RSB_WANT_OMPIO_SUPPORT */
+	}
 	#endif /* RSB_NUMERICAL_TYPE_INT */
 
 	#ifdef RSB_NUMERICAL_TYPE_CHAR
@@ -852,11 +1029,11 @@ full_scan:
 #else /* RSB_WANT_OMPIO_SUPPORT */
 	for (i=0; i<*nnz; i++)
 	{
-		int iI,iJ;
+		rsb_coo_idx_t iI,iJ;
 		double fv;
-		re += (rsb_zfscanf(fd,"%d %d %g\n",&iI,&iJ,&fv,NULL,ngzfd)==3);
-		(*IA)[i] = (rsb_coo_idx_t)iI;
-		(*JA)[i] = (rsb_coo_idx_t)iJ;
+		re += (rsb_zfscanf(fd,RSB_IJPC " %g\n",&iI,&iJ,&fv,NULL,ngzfd)==3);
+		(*IA)[i] = iI;
+		(*JA)[i] = iJ;
         	(*IA)[i]--;  /* adjust from 1-based to 0-based */
         	(*JA)[i]--;
 		(*cval)[i] = (char)fv;
@@ -874,11 +1051,11 @@ full_scan:
 #else /* RSB_WANT_OMPIO_SUPPORT */
 		for (i=0; i<*nnz; i++)
 		{
-			int iI,iJ;
+			rsb_coo_idx_t iI,iJ;
 			float rv,iv;
-			re += (rsb_zfscanf(fd,"%d %d %g %g\n",&iI,&iJ,&(rv),&(iv),ngzfd)==4);
-			(*IA)[i] = (rsb_coo_idx_t)iI;
-			(*JA)[i] = (rsb_coo_idx_t)iJ;
+			re += (rsb_zfscanf(fd,RSB_IJPC " %g %g\n",&iI,&iJ,&(rv),&(iv),ngzfd)==4);
+			(*IA)[i] = iI;
+			(*JA)[i] = iJ;
 	        	(*IA)[i]--;  /* adjust from 1-based to 0-based */
 	        	(*JA)[i]--;
 			(*zval)[i] = (rv + I * iv);
@@ -891,11 +1068,11 @@ full_scan:
 #else /* RSB_WANT_OMPIO_SUPPORT */
 		for (i=0; i<*nnz; i++)
 		{
-			int iI,iJ;
+			rsb_coo_idx_t iI,iJ;
 			float rv;
-			re += (rsb_zfscanf(fd,"%d %d %g\n",&iI,&iJ,&(rv),NULL,ngzfd)==3);
-			(*IA)[i] = (rsb_coo_idx_t)iI;
-			(*JA)[i] = (rsb_coo_idx_t)iJ;
+			re += (rsb_zfscanf(fd,RSB_IJPC " %g\n",&iI,&iJ,&(rv),NULL,ngzfd)==3);
+			(*IA)[i] = iI;
+			(*JA)[i] = iJ;
 	        	(*IA)[i]--;  /* adjust from 1-based to 0-based */
 	        	(*JA)[i]--;
 			(*zval)[i] = (rv + I * 0);
@@ -914,11 +1091,11 @@ full_scan:
 #else /* RSB_WANT_OMPIO_SUPPORT */
 		for (i=0; i<*nnz; i++)
 		{
-			int iI,iJ;
+			rsb_coo_idx_t iI,iJ;
 			double rv,iv;
-			re += (rsb_zfscanf(fd,"%d %d %lg %lg\n",&iI,&iJ,&(rv),&(iv),ngzfd)==4);
-			(*IA)[i] = (rsb_coo_idx_t)iI;
-			(*JA)[i] = (rsb_coo_idx_t)iJ;
+			re += (rsb_zfscanf(fd,RSB_IJPC " %lg %lg\n",&iI,&iJ,&(rv),&(iv),ngzfd)==4);
+			(*IA)[i] = iI;
+			(*JA)[i] = iJ;
 	        	(*IA)[i]--;  /* adjust from 1-based to 0-based */
 	        	(*JA)[i]--;
 			(*Zval)[i] = (rv + I * iv);
@@ -928,11 +1105,11 @@ full_scan:
 		else
 		for (i=0; i<*nnz; i++)
 		{
-			int iI,iJ;
+			rsb_coo_idx_t iI,iJ;
 			double rv;
-			re += (rsb_zfscanf(fd,"%d %d %lg\n",&iI,&iJ,&(rv),NULL,ngzfd)==3);
-			(*IA)[i] = (rsb_coo_idx_t)iI;
-			(*JA)[i] = (rsb_coo_idx_t)iJ;
+			re += (rsb_zfscanf(fd,RSB_IJPC " %lg\n",&iI,&iJ,&(rv),NULL,ngzfd)==3);
+			(*IA)[i] = iI;
+			(*JA)[i] = iJ;
 	        	(*IA)[i]--;  /* adjust from 1-based to 0-based */
 	        	(*JA)[i]--;
 			(*Zval)[i] = (rv + I * 0);
@@ -948,17 +1125,16 @@ full_scan:
 #else /* RSB_WANT_OMPIO_SUPPORT */
 	for (i=0; i<*nnz; i++)
 	{
-		int iI,iJ;
-		re += (rsb_zfscanf(fd,"%d %d\n",&iI,&iJ,NULL,NULL,ngzfd)==2);
-		(*IA)[i] = (rsb_coo_idx_t)iI;
-		(*JA)[i] = (rsb_coo_idx_t)iJ;
+		rsb_coo_idx_t iI,iJ;
+		re += (rsb_zfscanf(fd,RSB_IJPC "\n",&iI,&iJ,NULL,NULL,ngzfd)==2);
+		(*IA)[i] = iI;
+		(*JA)[i] = iJ;
         	(*IA)[i]--;  /* adjust from 1-based to 0-based */
         	(*JA)[i]--;
 		RSB_IO_VERBOSE_MSG(i,*nnz);
 	}
 #endif /* RSB_WANT_OMPIO_SUPPORT */
 	#endif /* RSB_NUMERICAL_TYPE_PATTERN */
-
 	if( is_lowerp || is_upperp )
 	{
 		rsb_flags_t flags = RSB_FLAG_NOFLAGS;
@@ -975,18 +1151,17 @@ full_scan:
 		rsb__fill_with_ones(*VA,otype,*nnz,1);
 	#endif /* RSB_NUMERICAL_TYPE_PATTERN */
 scan_done:
-	if (fd !=stdin)
+	if ( fd !=stdin || ngzfd )
 	{
-		// FIXME
 		if(ngzfd)
-			fclose(fd);
+			RSB_FCLOSE(ngzfd);
 		else
-			RSB_FCLOSE(fd);
+			fclose(fd);
 	}
 	if(re!=*nnz)
 	{
 		/* FIXME : this can happen when reading as double a complex matrix, now. */
-		RSB_STDERR("read only %zu out of %d matrix elements (incomplete or not a matrix file?)!\n",re,*nnz);
+		RSB_STDERR("read only %zu out of %zd matrix elements (incomplete or not a matrix file?)!\n",(rsb_printf_int_t)re,(rsb_printf_int_t)(*nnz));
 		goto err;
 	}
 
@@ -1014,10 +1189,15 @@ scan_done:
 
 		if(rsb__util_coo_check_if_triangle_non_empty(*IA,*JA,*nnz,RSB_FLAG_UPPER))
 		{
-			RSB_STDERR("#converting upper to lower triangle..\n");
+			if(wv)
+				RSB_STDERR("#converting upper to lower triangle..\n");
 			rsb__util_coo_upper_to_lower_symmetric(*IA,*JA,*nnz);
 			if(is_lower)
 				is_lower = RSB_BOOL_TRUE;
+			if( is_lowerp )
+				*is_lowerp = RSB_BOOL_TRUE;
+			if( is_upperp )
+				*is_upperp = RSB_BOOL_FALSE;
 		}
 
 		if(rsb__util_coo_check_if_triangle_non_empty(*IA,*JA,*nnz,RSB_FLAG_UPPER))
@@ -1026,7 +1206,7 @@ scan_done:
 			goto err;
 		}
 
-		errval = rsb__util_coo_check_if_has_diagonal_elements(*IA,*JA,*nnz,*m,&has_diagonal_elements);
+		errval = rsb__util_coo_check_if_has_diagonal_elements(*IA,*JA,*nnz,*m,&has_diagonal_elements,wv);
 		if(RSB_SOME_ERROR(errval))
 		{
 			RSB_STDERR("error while checking diagonal elements!\n");
@@ -1034,8 +1214,9 @@ scan_done:
 		}
 
 		if(!has_diagonal_elements)
+		if(wv)
 		{
-			RSB_STDERR("Input has missing elements on the diagonal.\n"); /* FIXME: emit this in a verbose mode only */
+			RSB_STDERR("Input has missing elements on the diagonal.\n");
 		}
 	}
 afterpmtxchecks:
@@ -1084,6 +1265,7 @@ err:
 ret:
 	RSB_DO_ERR_RETURN(errval);
 }
+#undef RSB_IJPC
 #endif /* RSB_WITH_MM */
 
 rsb_err_t rsb__do_util_get_matrix_dimensions(const char * filename, size_t * cols, size_t * rows, size_t * nnzp, rsb_flags_t*flagsp)
@@ -1126,6 +1308,7 @@ rsb_err_t rsb__do_util_get_matrix_dimensions(const char * filename, size_t * col
 	RSB_DO_ERR_RETURN(errval)
 }
 
+#ifdef RSB_OBSOLETE_QUARANTINE_UNUSED
 rsb_err_t rsb__util_mm_load_matrix_f_as_csr(const char *filename, rsb_nnz_idx_t ** INDX, rsb_coo_idx_t ** JA, void **VA, rsb_coo_idx_t *m, rsb_coo_idx_t *k , rsb_nnz_idx_t *nnz, rsb_type_t typecode, rsb_flags_t flags/*, rsb_bool_t *is_lowerp, rsb_bool_t *is_upperp*/)
 {
 	/**
@@ -1133,7 +1316,7 @@ rsb_err_t rsb__util_mm_load_matrix_f_as_csr(const char *filename, rsb_nnz_idx_t
 	 * FIXME : should optimize
 	 * */
 	rsb_err_t errval = RSB_ERR_NO_ERROR;
-       	struct rsb_coo_matrix_t coo;
+	struct rsb_coo_mtx_t coo;
 
 	RSB_DO_FLAG_ADD(flags,RSB_FLAG_WANT_BCSS_STORAGE);
 	*INDX = NULL;
@@ -1141,10 +1324,10 @@ rsb_err_t rsb__util_mm_load_matrix_f_as_csr(const char *filename, rsb_nnz_idx_t
 	*VA = NULL;
 	RSB_BZERO_P(&coo);
 	coo.typecode = typecode;
-       	errval = rsb_util_mm_load_coo_matrix(filename,&coo);
+       	errval = rsb__util_mm_load_coo_matrix(filename,&coo);
 	if(RSB_SOME_ERROR(errval))
 		goto err;
-	errval = rsb_util_sort_row_major_inner(coo.VA,coo.IA,coo.JA,coo.nnz,coo.nr,coo.nc,coo.typecode,flags);
+	errval = rsb__util_sort_row_major_inner(coo.VA,coo.IA,coo.JA,coo.nnz,coo.nr,coo.nc,coo.typecode,flags);
 	if(RSB_SOME_ERROR(errval))
 		goto err;
 	errval = rsb__allocate_csr_arrays_from_coo_sorted(coo.VA,coo.IA,coo.JA,coo.nnz,coo.nr,coo.nc,coo.typecode,VA,JA,INDX);
@@ -1155,7 +1338,9 @@ err:
 	rsb__destroy_coo_matrix_t(&coo);
 	RSB_DO_ERR_RETURN(errval)
 }
+#endif /* RSB_OBSOLETE_QUARANTINE_UNUSED */
 
+#ifdef RSB_OBSOLETE_QUARANTINE_UNUSED
 rsb_err_t rsb__util_mm_load_matrix_f_as_csc(const char *filename, rsb_nnz_idx_t ** INDX, rsb_coo_idx_t ** IA, void **VA, rsb_coo_idx_t *m, rsb_coo_idx_t *k , rsb_nnz_idx_t *nnz, rsb_type_t typecode, rsb_flags_t flags/*, rsb_bool_t *is_lowerp, rsb_bool_t *is_upperp*/)
 {
 	/** 
@@ -1163,13 +1348,13 @@ rsb_err_t rsb__util_mm_load_matrix_f_as_csc(const char *filename, rsb_nnz_idx_t
 	 * FIXME : should optimize
 	 * */
 	rsb_err_t errval = RSB_ERR_NO_ERROR;
-       	struct rsb_coo_matrix_t coo;
+	struct rsb_coo_mtx_t coo;
 
 	RSB_DO_FLAG_ADD(flags,RSB_FLAG_WANT_BCSS_STORAGE);
 	*INDX = NULL;*IA = NULL;*VA = NULL;
 	RSB_BZERO_P(&coo);
 	coo.typecode = typecode;
-       	errval = rsb_util_mm_load_coo_matrix(filename,&coo);
+       	errval = rsb__util_mm_load_coo_matrix(filename,&coo);
 	if(RSB_SOME_ERROR(errval))
 		goto err;
 	errval = rsb_util_sort_column_major(coo.VA,coo.IA,coo.JA,coo.nnz,coo.nr,coo.nc,coo.typecode,flags);
@@ -1183,15 +1368,17 @@ err:
 	rsb__destroy_coo_matrix_t(&coo);
 	RSB_DO_ERR_RETURN(errval)
 }
+#endif /* RSB_OBSOLETE_QUARANTINE_UNUSED */
 
-rsb_err_t rsb_util_mm_fill_arrays_for_csc(const char *filename, rsb_nnz_idx_t * INDX, rsb_coo_idx_t * IA, void *VA, rsb_type_t typecode, rsb_flags_t flags)
+#ifdef RSB_OBSOLETE_QUARANTINE_UNUSED
+rsb_err_t rsb__util_mm_fill_arrays_for_csc(const char *filename, rsb_nnz_idx_t * INDX, rsb_coo_idx_t * IA, void *VA, rsb_type_t typecode, rsb_flags_t flags)
 {
 	/** 
 	 * \ingroup gr_internals
 	 * FIXME : should optimize
 	 * */
 	rsb_err_t errval = RSB_ERR_NO_ERROR;
-       	struct rsb_coo_matrix_t coo;
+	struct rsb_coo_mtx_t coo;
 	rsb_nnz_idx_t *iINDX = NULL;
 	rsb_coo_idx_t *iIA = NULL;
 	void *iVA = NULL;
@@ -1202,7 +1389,7 @@ rsb_err_t rsb_util_mm_fill_arrays_for_csc(const char *filename, rsb_nnz_idx_t *
 
 	RSB_BZERO_P(&coo);
 	coo.typecode = typecode;
-       	errval = rsb_util_mm_load_coo_matrix(filename,&coo);
+       	errval = rsb__util_mm_load_coo_matrix(filename,&coo);
 	if(RSB_SOME_ERROR(errval))
 		goto err;
 	errval = rsb_util_sort_column_major(coo.VA,coo.IA,coo.JA,coo.nnz,coo.nr,coo.nc,coo.typecode,flags);
@@ -1218,8 +1405,9 @@ err:
 	rsb__destroy_coo_matrix_t(&coo);
 	RSB_DO_ERR_RETURN(errval)
 }
+#endif /* RSB_OBSOLETE_QUARANTINE_UNUSED */
 
-size_t rsb_sys_filesize(const char *filename)
+size_t rsb__sys_filesize(const char *filename)
 {
 	/**
 	 * file size, in bytes
