@@ -1,6 +1,6 @@
-/*                                                                                                                            
+/*
 
-Copyright (C) 2008-2015 Michele Martone
+Copyright (C) 2008-2022 Michele Martone
 
 This file is part of librsb.
 
@@ -21,7 +21,7 @@ If not, see <http://www.gnu.org/licenses/>.
 */
 /*! @file
  *  @author Michele Martone
- *  @brief Implementation of the library user interface.
+ *  @brief Implementation of the library user interface specified in rsb.h and rsb_types.h.
  */
 /*
  *  The user interface functions and data structures for this library implementations.
@@ -33,7 +33,7 @@ If not, see <http://www.gnu.org/licenses/>.
 #include <stdio.h>
 #include "rsb_do.h"
 
-RSB_INTERNALS_COMMON_HEAD_DECLS
+struct rsb_session_handle_t rsb_global_session_handle;
 
 #define RSB_INTERFACE_RETURN_MTX_ERRP(MTXAP,ERRVAL,ERRVALP) \
 	                                 RSB_INTERFACE_ENDCMD \
@@ -43,6 +43,8 @@ RSB_INTERNALS_COMMON_HEAD_DECLS
 /* #define RSB_INTERFACE_RETURN_ERR_SILENT(ERRVAL) RSB_INTERFACE_ENDCMD return (ERRVAL); */
 #define RSB_INTERFACE_RETURN_VAL(VAL)    RSB_INTERFACE_ENDCMD {return (VAL);}
 
+#define RSB_INITIALIZE_CHECK_MTX_ERRP(ERRVALP)	if( ! rsb__do_was_initialized() ) { RSB_ERROR(RSB_ERRM_UL); RSB_INTERFACE_RETURN_MTX_ERRP(NULL,RSB_ERR_UNSUPPORTED_OPERATION,ERRVALP) }
+
 /*!
  * \internal
  * This library, currently, can be used by only one master thread.
@@ -65,9 +67,16 @@ rsb_err_t rsb_lib_init(struct rsb_initopts * iop)
 	   Options may be specified also after \ref rsb_lib_init() by calling \ref rsb_lib_reinit().
 	   \n
 	   One may call #RSB_REINIT_SINGLE_VALUE_GET  with flag  #RSB_IO_WANT_IS_INITIALIZED_MARKER  to verify whether the library has been initialized or not.
+	   \n
+	   If the \c RSB_NUM_THREADS environment variable is set, \ref rsb_lib_init() uses it and sets the number of active threads, thus overriding what detected by the OpenMP runtime (e.g. \c OMP_NUM_THREADS).
 	  
 	   \param \rsb_io_str_msg
 	   \return \rsberrcodemsg
+
+	   An example snippet declaring an error variable accumulator at program's beginning:
+           \snippet examples/snippets.c Declare error codes variable
+           and initializing the library soon thereafter:
+           \snippet examples/snippets.c Initialize the library
 	   \see_lib_init
 	 */
 	rsb_err_t errval = RSB_ERR_NO_ERROR;
@@ -78,6 +87,7 @@ rsb_err_t rsb_lib_init(struct rsb_initopts * iop)
 
 /* @cond INNERDOC  */
 /* TODO: this is a "in development" function, not yet declared in rsb.h ; shall make it official when complete */
+#ifdef RSB_OBSOLETE_QUARANTINE_UNUSED
 rsb_err_t rsb__lib_get_info_str(int what, rsb_char_t* sbuf, size_t buflen)
 {
 	/* \see_lib_init */
@@ -86,16 +96,24 @@ rsb_err_t rsb__lib_get_info_str(int what, rsb_char_t* sbuf, size_t buflen)
 	errval = rsb__do_lib_get_info_str(what,sbuf,buflen);
 	RSB_INTERFACE_RETURN_ERR(errval)
 }
+#endif /* RSB_OBSOLETE_QUARANTINE_UNUSED */
 /* @endcond */
 
 rsb_err_t rsb_lib_set_opt(enum rsb_opt_t iof, const void*iop)
 {
 	/*!
+	   \ingroup rsb_doc_library rsb_doc_rsb
+
 	 Sets value of a library option.
+	 \n
  	 A value specified by the request flag \c iof  will be fetched from \c *iop and will be used to update the selected option in the library internal state.
 
 	 \rsb_iof_param_msg
 	 \rsb_iop_out_param_msg
+
+	 Example snip:
+	 \snippet examples/snippets.c Setting a single optional library parameter
+
 	 \see \rsb_iof_macros
 	 \see_lib_init
 	 */
@@ -108,7 +126,10 @@ rsb_err_t rsb_lib_set_opt(enum rsb_opt_t iof, const void*iop)
 rsb_err_t rsb_lib_get_opt(enum rsb_opt_t iof, void*iop)
 {
 	/*!
+	   \ingroup rsb_doc_library rsb_doc_rsb
+
 	 Gets value of a library option.
+	 \n
  	 A value specified by the request flag \c iof  will be fetched from the library internal state and \c *iop will be updated accordingly.
 
 	 \rsb_iof_param_msg
@@ -128,6 +149,7 @@ rsb_err_t rsb_lib_set_opt_str(const rsb_char_t* opnp, const rsb_char_t* opvp)
 	   \ingroup rsb_doc_library rsb_doc_rsb
 
 	   Specifies individual library options in order to fine-tune the library behaviour.
+	   \n
 	   Both the option name and the value shall be expressed as strings, identical to their preprocessor identifiers (see #rsb_opt_t ).
 	   The \c opnp string will be translated internally to the corresponding request flag values, and the passed value will be parsed out of the \c opvp string.
 	   \n
@@ -136,6 +158,8 @@ rsb_err_t rsb_lib_set_opt_str(const rsb_char_t* opnp, const rsb_char_t* opvp)
 	   \param \rsb_io_str_msg_opvp
 	   \return \rsberrcodemsg
 
+           \snippet examples/snippets.c rsb_lib_reinit__rsb_lib_set_opt_str_snip
+
 	   \see_lib_init
 	 */
 	rsb_err_t errval = RSB_ERR_NO_ERROR;
@@ -162,6 +186,9 @@ rsb_err_t rsb_lib_reinit(struct rsb_initopts * iop)
 	   
 	   \param \rsb_io_str_msg
 	   \return \rsberrcodemsg
+
+           \snippet examples/snippets.c rsb_lib_reinit__rsb_lib_set_opt_str_snip
+
 	   \see_lib_init
 	 */
 	rsb_err_t errval = RSB_ERR_NO_ERROR;
@@ -189,9 +216,17 @@ rsb_err_t rsb_lib_exit(struct rsb_initopts * iop)
 	   terminate program execution (rather than forcing a new initialization with \ref rsb_lib_init()).
 	   \n
 	   Parameter  \c iop  is reserved for future use; for now it is safe to pass #RSB_NULL_EXIT_OPTIONS.
+	   \n
+	   It should be safe to call \ref rsb_lib_exit() more than once.
+	   \n
 
 	   \param \rsb_io_str_msg
 	   \return \rsberrcodemsg
+
+	   An example snippet declaring an error variable accumulator at program's beginning:
+           \snippet examples/snippets.c Declare error codes variable
+           and finalizing the library at program's end:
+           \snippet examples/snippets.c Finalize the library
 	   \see_lib_init
 	 */
 	rsb_err_t errval = RSB_ERR_NO_ERROR;
@@ -210,16 +245,25 @@ struct rsb_mtx_t * rsb_mtx_alloc_from_coo_const(const void *VA, const rsb_coo_id
 	   \param \rsb_ro_va_ia_ja_desc_msg
 	   \param \rsb_nnzA_inp_param_msg
 	   \param \rsb_type_param_msg
-	   \param \rsb_nrcows_A_sparse_inp_param_msg
+	   \param \rsb_nrcows_A_sparse_inp_param_msg If any of \rsb_nrA or \rsb_ncA is zero, it will be detected on the basis of the \c IA and \c JA arrays and \rsb_flagsA.
 	   \param \rsb_nrbows_A_sparse_inp_param_msg
 	   \param \rsb_flagsa_coc_param_msg
 	   \param \rsb_errvp_inp_param_msg
 	   \return \rsbmtxpmessage
-	   \see_lib_alloc
+
+	   Example snip:
+	   \snippet examples/snippets.c Allocate matrix without error flags check
+	   And another, with duplicate sum flags:
+	   \snippet examples/snippets.c Allocate matrix with error flags check
+	   And yet another, allocating a triangular matrix:
+	   \snippet examples/snippets.c Allocate a matrix with triangular flags
+
+           \see_lib_alloc
 	 */
 	rsb_err_t errval = RSB_ERR_NO_ERROR;
 	struct rsb_mtx_t * mtxAp = NULL;
 	RSB_INTERFACE_PREAMBLE
+	RSB_INITIALIZE_CHECK_MTX_ERRP(errvalp);
 	mtxAp = rsb__do_mtx_alloc_from_coo_const(VA,IA,JA,nnzA,typecode,nrA,ncA,brA,bcA,flagsA,&errval);
 	RSB_INTERFACE_RETURN_MTX_ERRP(mtxAp,errval,errvalp);
 }
@@ -247,6 +291,7 @@ struct rsb_mtx_t * rsb_mtx_alloc_from_coo_inplace(void *VA, rsb_coo_idx_t * IA,
 	rsb_err_t errval = RSB_ERR_NO_ERROR;
 	struct rsb_mtx_t * mtxAp = NULL;
 	RSB_INTERFACE_PREAMBLE
+	RSB_INITIALIZE_CHECK_MTX_ERRP(errvalp);
 	mtxAp = rsb__do_mtx_alloc_from_coo_inplace(VA,IA,JA,nnzA,typecode,nrA,ncA,brA,bcA,flagsA,&errval);
 	RSB_INTERFACE_RETURN_MTX_ERRP(mtxAp,errval,errvalp);
 }
@@ -263,6 +308,8 @@ struct rsb_mtx_t * rsb_mtx_free(struct rsb_mtx_t * mtxAp)
 	   \param \rsb_mtxt_inp_param_msg_a
 	   \return \rsb_ret_null
 
+           Example freeing a sparse matrix:
+           \snippet examples/snippets.c Free a sparse matrix
 	   \see_lib_alloc
 	 */
 	struct rsb_mtx_t * mtxBp = NULL;
@@ -285,15 +332,18 @@ rsb_err_t rsb_mtx_clone(struct rsb_mtx_t ** mtxBpp, rsb_type_t typecode, rsb_tra
 	   This new structure will be completely separated and independent from the original one.
 	   \n
 	   Examples:
-	   \code
+	 */
+	 /**
+\code{.c}
 // will clone the matrix exactly
 errval = rsb_mtx_clone(&mtxBp,RSB_NUMERICAL_TYPE_SAME_TYPE,RSB_TRANSPOSITION_N,NULL,mtxAp,RSB_FLAG_IDENTICAL_FLAGS);
 // will clone the transpose of the matrix
 errval = rsb_mtx_clone(&mtxBp,RSB_NUMERICAL_TYPE_SAME_TYPE,RSB_TRANSPOSITION_T,NULL,mtxAp,RSB_FLAG_IDENTICAL_FLAGS);
 // will clone the lower triangle of the matrix
 errval = rsb_mtx_clone(&mtxBp,RSB_NUMERICAL_TYPE_SAME_TYPE,RSB_TRANSPOSITION_N,NULL,mtxAp,RSB_FLAG_TRIANGULAR|RSB_FLAG_LOWER);
-	   \endcode
-	  
+\endcode
+	 */
+	 /**
 	   \param \rsb_mtxtpp_inp_param_msg_b If \c *mtxBpp==NULL, a fresh clone will be assigned there; if not, the existing matrix structure will be freed and allocated to host the new one. The case \c *mtxBpp==mtxAp is supported.
 	   \param \rsb_type_o_param_msg
 	   \param \rsb_transa_inp_param_msg
@@ -302,12 +352,15 @@ errval = rsb_mtx_clone(&mtxBp,RSB_NUMERICAL_TYPE_SAME_TYPE,RSB_TRANSPOSITION_N,N
 	   \param \rsb_flags_stru_fla_msg
 	   \return \rsberrcodemsg
 
+	   Example snip:
+	   \snippet examples/snippets.c Clone and transpose a sparse matrix
+
 	   \see_lib_alloc
 	 */
 	/* FIXME: what if RSB_FLAG_EXTERNALLY_ALLOCATED_ARRAYS ? */
 	rsb_err_t errval = RSB_ERR_NO_ERROR;
 	RSB_INTERFACE_PREAMBLE
-	errval = rsb__clone(mtxBpp,typecode,transA,alphap,mtxAp,flags);
+	errval = rsb__mtx_clone(mtxBpp,typecode,transA,alphap,mtxAp,flags);
 	RSB_INTERFACE_RETURN_ERR(errval)
 }
 
@@ -407,8 +460,15 @@ rsb_err_t rsb_spmv_nt(const void *alphap, const struct rsb_mtx_t * mtxAp, const
 		rsb_spmv(RSB_TRANSPOSITION_T,alphap,mtxAp,x2p,incX,betap,y2p,incY);
 	RSB_INTERFACE_RETURN_ERR(errval)
 }
+#endif
 
-rsb_err_t rsb_spmv_ata(const void *alphap, const struct rsb_mtx_t * mtxAp, const void * Xp, rsb_coo_idx_t incX, const void * betap, void * Yp, rsb_coo_idx_t incY)
+#ifdef RSB_OBSOLETE_QUARANTINE_UNUSED
+/* Postponed... (from rsb.h) */
+rsb_err_t rsb_spata(const void *alphap, const struct rsb_mtx_t * mtxAp, const void * Xp, rsb_coo_idx_t incX, const void * betap, void * Yp, rsb_coo_idx_t incY);
+#endif /* RSB_OBSOLETE_QUARANTINE_UNUSED */
+#ifdef RSB_OBSOLETE_QUARANTINE_UNUSED
+/* Postponed... */
+rsb_err_t rsb_spata(const void *alphap, const struct rsb_mtx_t * mtxAp, const void * Xp, rsb_coo_idx_t incX, const void * betap, void * Yp, rsb_coo_idx_t incY)
 {
 	/*!
 	   \ingroup rsb_doc_matrix_operations rsb_doc_rsb
@@ -427,12 +487,15 @@ rsb_err_t rsb_spmv_ata(const void *alphap, const struct rsb_mtx_t * mtxAp, const
 	   \warning \rsb_warn_unimplemented_msg
 	   \warning \rsb_warn_untested_msg
 	 */
+	/* FIXME: untested; details to be finished ! */
 	rsb_err_t errval = RSB_ERR_UNIMPLEMENTED_YET;
 	RSB_INTERFACE_PREAMBLE
+	errval = rsb__do_spata(alphap, mtxAp, Xp, incX, betap, Yp, incY);
 	RSB_INTERFACE_RETURN_ERR(errval)
-	// FIXME: this is only a placeholder, waiting for a combined implementation.
 }
+#endif /* RSB_OBSOLETE_QUARANTINE_UNUSED */
 
+#if 0
 rsb_err_t rsb_spmv_power(rsb_trans_t transA, const void *alphap, const struct rsb_mtx_t * mtxAp,  rsb_int_t exp, const void * Xp, rsb_coo_idx_t incX, const void * betap, void * Y, rsb_coo_idx_t incY)
 {
 	/*!
@@ -485,6 +548,11 @@ rsb_err_t rsb_spmv(rsb_trans_t transA, const void *alphap, const struct rsb_mtx_
 	   \param \rsb_y_out_param_msg
 	   \param \rsb_incy_inp_param_msg
 	   \return \rsberrcodemsg
+
+	   Example snip:
+	   \snippet examples/snippets.c Multiply a sparse matrix by a dense vector
+
+	   \rsb_librsbpp_env
 	   \see_lib_spmx
 	 */
 	rsb_err_t errval = RSB_ERR_NO_ERROR;
@@ -504,7 +572,7 @@ rsb_err_t rsb_spmv_sa(const struct rsb_mtx_t * mtxAp, const void * Xp, void * Yp
 	 * */
 	if(!alphap || !mtxAp)
 		return RSB_ERR_BADARGS;
-	return rsb_do_spmv_general(transA,alphap,mtxTp,Xp,1,NULL,Yp,1,RSB_OP_FLAG_DEFAULT RSB_DEFAULT_OUTER_NRHS_SPMV_ARGS);
+	return rsb__do_spmv_general(transA,alphap,mtxTp,Xp,1,NULL,Yp,1,RSB_OP_FLAG_DEFAULT RSB_DEFAULT_OUTER_NRHS_SPMV_ARGS);
 }
 #endif
 
@@ -527,6 +595,10 @@ rsb_err_t rsb_spsv(rsb_trans_t transT, const void * alphap, const struct rsb_mtx
 	   \param \rsb_incy_inp_param_msg
 	   \return \rsberrcodemsg
 	   \rsb_spsv_no_zero
+
+	   Example backsolving a triangular system:
+	   \snippet examples/snippets.c Backsolve a triangular system
+
 	   \see_lib_spsx
 	 */
 	rsb_err_t errval = RSB_ERR_NO_ERROR;
@@ -559,7 +631,7 @@ rsb_err_t rsb_spmv_uxux(const struct rsb_mtx_t * mtxAp, const void * Xp, void *
 	 * */
 	if(!alphap || !betap)
 		return RSB_ERR_BADARGS;
-	return rsb_do_spmv_general(transA,alphap,mtxAp,Xp,1,betap,Yp,1,RSB_OP_FLAG_DEFAULT RSB_DEFAULT_OUTER_NRHS_SPMV_ARGS);
+	return rsb__do_spmv_general(transA,alphap,mtxAp,Xp,1,betap,Yp,1,RSB_OP_FLAG_DEFAULT RSB_DEFAULT_OUTER_NRHS_SPMV_ARGS);
 }
 #endif
 
@@ -657,11 +729,61 @@ rsb_err_t rsb_coo_sort(void *VA, rsb_coo_idx_t * IA, rsb_coo_idx_t * JA, rsb_nnz
 	errval = rsb__util_sort_row_major_buffered(VA,IA,JA,nnzA,nrA,ncA,typecode,flags,NULL,0);
 #else
 	/* This is not the default, well tested. */
-	errval = rsb_util_sort_row_major_inner(VA,IA,JA,nnzA,nrA,ncA,typecode,flagsA);
+	errval = rsb__util_sort_row_major_inner(VA,IA,JA,nnzA,nrA,ncA,typecode,flagsA);
 #endif
 	RSB_INTERFACE_RETURN_ERR(errval)
 }
 
+rsb_err_t rsb_coo_cleanup(rsb_coo_idx_t* nnzp, void* VA, rsb_coo_idx_t* IA, rsb_coo_idx_t* JA, rsb_nnz_idx_t nnzA, rsb_coo_idx_t nrA, rsb_coo_idx_t ncA, rsb_type_t typecode, rsb_flags_t flagsA )
+{
+	/*!
+	   \ingroup gr_util rsb_doc_rsb
+
+	   Compacts the given COO input arrays representing a sparse matrix \f$A\f$.
+	   Will either sum together duplicates or use the last one, depending on whether #RSB_FLAG_DUPLICATES_KEEP_LAST or #RSB_FLAG_DUPLICATES_SUM is present in flagsA.
+
+	   It is important that the input is sorted and flagsA shall contain RSB_FLAG_SORTED_INPUT, otherwise the algorithm's complexity will be quadratic.
+
+	   \param nnzp  Pointer to the number of nonzeroes after the cleanup. 
+	   \param \rsb_wr_va_ia_ja_desc_msg
+	   \param \rsb_nnzA_inp_param_msg
+	   \param \rsb_nrcows_A_sparse_inp_param_msg
+	   \param \rsb_type_param_msg
+	   \param \rsb_flagsa_inp_param_msg If unsure, use #RSB_FLAG_NOFLAGS.
+	   \return \rsberrcodemsg
+	   \see_lib_util
+	   \see rsb_coo_sort
+
+	   \warning	This is an experimental librsb-1.3 function.
+
+	   \note By invoking with swapped \c IA and \c JA (and swapping \c nrA and \c ncA as well) one can obtain column major order.
+
+	 */
+	 /**
+	   Examples:
+
+	   \snippet examples/snippets.c COO cleanup 1
+
+	   \snippet examples/snippets.c COO cleanup 2
+	 */
+
+	/* TODO: "By invoking" .. in doc/Doxyfile. */
+	/* Cleans up and compacts the given COO input arrays representing a sparse matrix \f$A\f$.  Will delete any nonzero non compliant with the given flags. */
+	/* Relevant flags are: RSB_FLAG_UNIT_DIAG_IMPLICIT, RSB_FLAG_LOWER_TRIANGULAR, RSB_FLAG_UPPER_TRIANGULAR, RSB_FLAG_DISCARD_ZEROS. */
+	rsb_err_t errval = RSB_ERR_NO_ERROR;
+	rsb_nnz_idx_t nnz = 0;
+
+	RSB_INTERFACE_PREAMBLE
+	/* rsb_coo_idx roff=0,coff=0;
+	errval = rsb__do_cleanup_nnz(VA,IA,JA,nnzA,roff,coff,nrA,ncA,nnzp,typecode,flagsA); */
+	/* TODO: if sorted shall look for duplicates */
+	if( nnzp )
+		nnz = rsb__weed_out_duplicates(IA,JA,VA,nnzA,typecode,flagsA);
+	RSB_SET_IF_NOT_NULL(nnzp,nnz);
+	/* TODO: shall accumulate duplicates */
+	RSB_INTERFACE_RETURN_ERR(errval)
+}
+
 rsb_err_t rsb_file_mtx_get_dims(const char * filename, rsb_coo_idx_t* nrp, rsb_coo_idx_t *ncp, rsb_coo_idx_t *nzp, rsb_flags_t*flagsp)
 {
 	/*!
@@ -674,10 +796,13 @@ rsb_err_t rsb_file_mtx_get_dims(const char * filename, rsb_coo_idx_t* nrp, rsb_c
 	   \param \rsb_flagsp_inp_param_msg
 	   \return \rsberrcodemsg If read dimensions are illegal (see #rsb_coo_idx_t,#rsb_nnz_idx_t), #RSB_ERR_LIMITS will be returned.
 
+	   Example getting dimensions of a sparse matrix stored in a Matrix Market file:
+	   \snippet examples/snippets.c Get dimensions of sparse matrix stored in Matrix Market file
+
 	   \rsb_matrixmarketonlynote_m
+	   \note Upper/lower flags will not be reported; hermitiannes do.
 	   \see_lib_get
 	*/
-	/* TODO: do we detect/read hermitiann'ess ? */
 	rsb_err_t errval = RSB_ERR_NO_ERROR;
 	RSB_INTERFACE_PREAMBLE
 	errval = rsb__do_file_mtx_get_dims(filename, nrp, ncp, nzp, flagsp);
@@ -712,13 +837,19 @@ rsb_err_t rsb_strerror_r(rsb_err_t errval, rsb_char_t * buf, size_t buflen)
 	   \ingroup rsb_doc_error_handling rsb_doc_rsb
 
 	   Writes a textual description of an error code in the specified string buffer.
-	   No more than buflen characters will be written (comprehensive of the terminting \c NUL character).
+	   No more than buflen characters will be written (comprehensive of the terminating \c NUL character).
 	  
 	   \param \rsb_errval_inp_param_msg
 	   \param \rsb_buf_inp_param_msg
 	   \param \rsb_buflen_inp_param_msg
 
 	   \return \rsberrcodemsg
+
+           Examples:
+           \snippet examples/snippets.c Copy error message to string
+	or
+           \snippet examples/snippets.c snip__rsb_strerror_r
+
 	   \see_lib_error
 	 */
 	RSB_INTERFACE_PREAMBLE
@@ -732,7 +863,7 @@ rsb_err_t rsb_mtx_upd_vals(struct rsb_mtx_t * mtxAp, enum rsb_elopf_t elop_flags
 	   \ingroup rsb_doc_matrix_handling rsb_doc_rsb
 
 	   \f$ A \leftarrow op (A,\Omega) \f$
-	   Updates the matrix \f$A\f$ by applying either a rowwise or an elemental operation \f$op\f$, which is determined by \c elop_flags.
+	   Updates the matrix \f$A\f$ by applying either a row-wise or an elemental operation \f$op\f$, which is determined by \c elop_flags.
 	   If an unary operation is selected, \c omegap can be \c NULL.
 
 	   \param \rsb_mtxt_inp_param_msg_a
@@ -740,6 +871,9 @@ rsb_err_t rsb_mtx_upd_vals(struct rsb_mtx_t * mtxAp, enum rsb_elopf_t elop_flags
 	   \param \rsb_omega_inp_param_msg
 	   \return \rsberrcodemsg
 
+           Example snip:
+	   \snippet examples/snippets.c snip__rsb_mtx_upd_vals
+
 	   \see_lib_set
 	 */
 	rsb_err_t errval = RSB_ERR_NO_ERROR;
@@ -793,8 +927,12 @@ rsb_err_t rsb_mtx_get_vals(const struct rsb_mtx_t * mtxAp, void * VA, const rsb_
 	   \param \rsb_flags_getv_inp_param_msg
 	   \return \rsberrcodemsg
 
+	   Example snip:
+           \snippet examples/snippets.c snip__rsb_mtx_get_vals
+
            \see_lib_get
 	 */
+
 	/* may return an ...UNFINALIZED... error here ... */
 	/* TODO: could document better error behaviour (e.g.: what if all updated except one ? ) */
 	rsb_err_t errval = RSB_ERR_NO_ERROR;
@@ -818,6 +956,8 @@ rsb_err_t rsb_file_mtx_save(const struct rsb_mtx_t * mtxAp, const rsb_char_t * f
 
 	   \rsb_matrixmarketonlynote_m
 
+           Example, printing a matrix to standard output:
+           \snippet examples/snippets.c Print a matrix to standard output
            \see_lib_info
 	 */
 	rsb_err_t errval = RSB_ERR_NO_ERROR;
@@ -841,6 +981,10 @@ rsb_err_t rsb_file_vec_save(const rsb_char_t * filename, rsb_type_t typecode, co
 	   \return \rsberrcodemsg
 
 	   \rsb_matrixmarketonlynote_v
+
+	   Example printing to standard output:
+	   \snippet examples/snippets.c Print to stdout an nrA-long numerical vector
+
            \see_lib_info
 	 */
 	rsb_err_t errval = RSB_ERR_NO_ERROR;
@@ -863,6 +1007,9 @@ rsb_err_t rsb_file_vec_load(const rsb_char_t * filename, rsb_type_t typecode, vo
 	   \param \rsb_yvlp_param_msg
 	   \return \rsberrcodemsg
 
+	   Example loading vector matrix from file
+	   \snippet examples/snippets.c Load vector matrix from file
+
 	   \rsb_matrixmarketonlynote_v
            \see_lib_info
 	 */
@@ -878,6 +1025,7 @@ struct rsb_mtx_t * rsb_file_mtx_load(const rsb_char_t * filename, rsb_flags_t fl
 	   \ingroup rsb_doc_input_output rsb_doc_rsb
 
 	   Loads a sparse matrix from the specified matrix file, assembling it in the format specified by \rsb_flags, using the numerical type representation as specified by the user.
+	   \rsb_extra_internal_verbosity
 
 	   \param \rsb_filename_inp_param_msg
 	   \param \rsb_flagsa_inp_param_msg
@@ -886,11 +1034,15 @@ struct rsb_mtx_t * rsb_file_mtx_load(const rsb_char_t * filename, rsb_flags_t fl
 	   \return \rsbmtxpmessage
 
 	   \rsb_matrixmarketonlynote_m
+
+           Example loading a matrix from a Matrix Market file:
+           \snippet examples/snippets.c Load a matrix from Matrix Market file
 	   \see_lib_info
 	 */
 	struct rsb_mtx_t * mtxAp = NULL;
 	rsb_err_t errval = RSB_ERR_NO_ERROR;
 	RSB_INTERFACE_PREAMBLE
+	RSB_INITIALIZE_CHECK_MTX_ERRP(errvalp);
 	mtxAp = rsb__dodo_load_matrix_file_as_matrix_market(filename, flagsA, typecode, &errval);
 	RSB_INTERFACE_RETURN_MTX_ERRP(mtxAp,errval,errvalp);
 }
@@ -917,6 +1069,9 @@ struct rsb_mtx_t * rsb_sppsp(rsb_type_t typecode, rsb_trans_t transA, const void
 	   \param \rsb_errvp_inp_param_msg
 	   \return \rsbmtxpmessage
 
+	   Example snip:
+	   \snippet examples/snippets.c snip__rsb_sppsp
+
 	   \see_lib_gemm
 
 	   \warning \rsb_warn_not_th_tested_msg
@@ -925,6 +1080,7 @@ struct rsb_mtx_t * rsb_sppsp(rsb_type_t typecode, rsb_trans_t transA, const void
 	rsb_err_t errval = RSB_ERR_NO_ERROR;
 	struct rsb_mtx_t * mtxCp = NULL;
 	RSB_INTERFACE_PREAMBLE
+	RSB_INITIALIZE_CHECK_MTX_ERRP(errvalp);
 	mtxCp = rsb__do_matrix_sum(typecode,transA,alphap,mtxAp,transB,betap,mtxBp,&errval);
 	RSB_INTERFACE_RETURN_MTX_ERRP(mtxCp,errval,errvalp);
 }
@@ -951,6 +1107,9 @@ struct rsb_mtx_t * rsb_spmsp(rsb_type_t typecode, rsb_trans_t transA, const void
 	   \param \rsb_errvp_inp_param_msg
 	   \return \rsbmtxpmessage
 
+	   Example snip:
+	   \snippet examples/snippets.c snip__rsb_spmsp
+
 	   \warning Parameters \c alphap,betap,transA,transB  are not yet taken in consideration. The following defaults are valid: \f$\alpha=1.0\f$ and \f$\beta=1.0\f$, and \c transA=transB=#RSB_TRANSPOSITION_N.
 
 	   \see_lib_gemm
@@ -960,6 +1119,7 @@ struct rsb_mtx_t * rsb_spmsp(rsb_type_t typecode, rsb_trans_t transA, const void
 	struct rsb_mtx_t * mtxCp = NULL;
 	rsb_err_t errval = RSB_ERR_NO_ERROR;
 	RSB_INTERFACE_PREAMBLE
+	RSB_INITIALIZE_CHECK_MTX_ERRP(errvalp);
 	mtxCp = rsb__do_matrix_mul(typecode,transA,alphap,mtxAp,transB,betap,mtxBp,&errval);
 	RSB_INTERFACE_RETURN_MTX_ERRP(mtxCp,errval,errvalp);
 }
@@ -979,9 +1139,13 @@ rsb_err_t rsb_mtx_add_to_dense(const void *alphap, const struct rsb_mtx_t * mtxA
 	   \param \rsb_rowmajor_B_inp_param_msg
 	   \param \rsb_dmtx_abi_param_msg_b
 	   \return \rsberrcodemsg
-	   \warning \rsb_warn_not_th_tested_msg
+
+           Example snip:
+	   \snippet examples/snippets.c snip__rsb_mtx_add_to_dense
 
 	   \note Please note that it suffices to 'transpose' \c Bp's description parameters to get \f$A\f$ transposed summed in.
+	   \note Symmetry is currently not expanded.
+	   \note Threaded, for large enough matrices.
 	   \see_lib_gemm
 	 */
 	/* TODO: add transA */
@@ -996,17 +1160,21 @@ rsb_trans_t rsb_psblas_trans_to_rsb_trans(const char psbtrans)
 	/*!
 	   \ingroup rsb_doc_misc rsb_doc_rsb
 	
-	    "Translates" a PSBLAS transposition value character to a \librsb one. 
+	    Translate a PSBLAS transposition value character to a \librsb one.
 	    \n
 	    See the PSBLAS library website/documentation for valid input values.
 
 	   \param \rsb_psb_trans_inp_param_msg 
 	   \return A valid transposition code; that is #RSB_TRANSPOSITION_N for 'N', #RSB_TRANSPOSITION_T for 'T', RSB_TRANSPOSITION_C for 'C',  (See \ref matrix_transposition_flags_section).
+
+           Example snip:
+	   \snippet examples/snippets.c main_rsb_psblas_trans_to_rsb_trans
+
 	   \see_lib_psblas
 	 */
-	rsb_trans_t rsbtrans = RSB_INVALID_TRANS_CHAR;
+	rsb_trans_t rsbtrans = RSB_TRANSPOSITION_INVALID;
 	RSB_INTERFACE_PREAMBLE
-	rsbtrans = rsb_do_psblas_trans_to_rsb_trans(psbtrans);
+	rsbtrans = rsb__do_psblas_trans_to_rsb_trans(psbtrans);
 	RSB_INTERFACE_RETURN_VAL(rsbtrans)
 }
 
@@ -1032,6 +1200,7 @@ struct rsb_mtx_t * rsb_mtx_alloc_from_csr_const(const void *VA, const rsb_coo_id
 	rsb_err_t errval = RSB_ERR_NO_ERROR;
 	struct rsb_mtx_t * mtxAp = NULL;
 	RSB_INTERFACE_PREAMBLE
+	RSB_INITIALIZE_CHECK_MTX_ERRP(errvalp);
 	mtxAp = rsb__do_mtx_alloc_from_csr_const(VA,RP,JA,nnzA,typecode,nrA,ncA,brA,bcA,flagsA,&errval);
 	RSB_INTERFACE_RETURN_MTX_ERRP(mtxAp,errval,errvalp);
 }
@@ -1052,12 +1221,16 @@ struct rsb_mtx_t * rsb_mtx_alloc_from_csc_const(const void *VA, const rsb_coo_id
 	   \param \rsb_errvp_inp_param_msg
 	   \return \rsbmtxpmessage
 	   \see_lib_alloc
+
+           Example:
+           \snippet examples/snippets.c snip__rsb_mtx_alloc_from_csc_const
 	 */
 	// FIXME: flags and index and alloc mangling, here 
 	// FIXME: UNTESTED, AND NNZ<M ?
 	rsb_err_t errval = RSB_ERR_NO_ERROR;
 	struct rsb_mtx_t * mtxAp = NULL;
 	RSB_INTERFACE_PREAMBLE
+	RSB_INITIALIZE_CHECK_MTX_ERRP(errvalp);
        	mtxAp = rsb__do_mtx_alloc_from_csc_const(VA,IA,CP,nnzA,typecode,nrA,ncA,brA,bcA,flagsA,&errval);
 	RSB_INTERFACE_RETURN_MTX_ERRP(mtxAp,errval,errvalp);
 }
@@ -1084,6 +1257,7 @@ struct rsb_mtx_t * rsb_mtx_alloc_from_csr_inplace (void *VA, rsb_nnz_idx_t * RP,
 	struct rsb_mtx_t * mtxAp = NULL;
 	rsb_err_t errval = RSB_ERR_NO_ERROR;
 	RSB_INTERFACE_PREAMBLE
+	RSB_INITIALIZE_CHECK_MTX_ERRP(errvalp);
 	mtxAp = rsb__do_mtx_alloc_from_csr_inplace (VA, RP, JA, nnzA, typecode, nrA, ncA, brA, bcA, flagsA, &errval );
 	RSB_INTERFACE_RETURN_MTX_ERRP(mtxAp,errval,errvalp);
 }
@@ -1102,6 +1276,10 @@ rsb_err_t rsb_mtx_switch_to_csr(struct rsb_mtx_t * mtxAp, void ** VAp, rsb_coo_i
 
 	   \note \rsb_note_switch_in_place
 	   \warning \rsb_warn_not_th_tested_msg
+
+           Example:
+           \snippet examples/snippets.c snip__rsb_mtx_switch_to_csr
+
 	   \see_lib_conv
 	 */
 	rsb_err_t errval = RSB_ERR_NO_ERROR;
@@ -1184,6 +1362,10 @@ rsb_err_t rsb_mtx_get_rows_sparse(rsb_trans_t transA, const void * alphap, const
 	   \param \rsb_transa_inp_param_msg
 	   \param \rsb_flags_getrs_inp_param_msg
 	   \return \rsberrcodemsg
+	   
+           Example snip:
+	   \snippet examples/snippets.c snip__rsb_mtx_get_rows_sparse
+
 	   \see_lib_get
          */
 	rsb_err_t errval = RSB_ERR_NO_ERROR;
@@ -1192,14 +1374,14 @@ rsb_err_t rsb_mtx_get_rows_sparse(rsb_trans_t transA, const void * alphap, const
 	RSB_INTERFACE_RETURN_ERR(errval)
 }
 
-rsb_err_t rsb_mtx_get_coo_block(const struct rsb_mtx_t * mtxAp, void* VA, rsb_coo_idx_t * IA, rsb_coo_idx_t * JA, rsb_coo_idx_t frA, rsb_coo_idx_t lrA, rsb_coo_idx_t fcA, rsb_coo_idx_t lcA, rsb_coo_idx_t * IREN, rsb_coo_idx_t * JREN, rsb_nnz_idx_t *rnzp, rsb_flags_t flags )
+rsb_err_t rsb_mtx_get_coo_block(const struct rsb_mtx_t * mtxAp, void* VA, rsb_coo_idx_t * IA, rsb_coo_idx_t * JA, rsb_coo_idx_t frA, rsb_coo_idx_t lrA, rsb_coo_idx_t fcA, rsb_coo_idx_t lcA, const rsb_coo_idx_t * IREN, const rsb_coo_idx_t * JREN, rsb_nnz_idx_t *rnzp, rsb_flags_t flags )
 {
 	/*!
 	   \ingroup rsb_doc_matrix_conversion rsb_doc_rsb
-
 	   Writes in COO format the specified submatrix.
+	   \n
 	   Works in two stages: first the user invokes it with \c VA,IA,JA set to \c NULL  to get \c *rnzp.
-	   Then the the \c VA,IA,JA arrays can be allocated, and the function called again, this time with \c rnzp=NULL but the \c VA,IA,JA arrays pointers non \c NULL (or at least, one of them).
+	   Then the \c VA,IA,JA arrays can be allocated, and the function called again, this time with \c rnzp=NULL but the \c VA,IA,JA arrays pointers non \c NULL (or at least, one of them).
 
 	   \param \rsb_mtxt_inp_param_msg_a
 	   \param \rsb_wr_va_ia_ja_desc_msg
@@ -1209,16 +1391,22 @@ rsb_err_t rsb_mtx_get_coo_block(const struct rsb_mtx_t * mtxAp, void* VA, rsb_co
 	   \param \rsb_inp_rnz_msg
 	   \param \rsb_flags_getcb_inp_param_msg
 	   \return \rsberrcodemsg
+
 	   Examples:
-	   \code
+           \snippet examples/snippets.c Extract one sparse matrix block
+*/
+/**
+	   And other examples:
+\code{.c}
 // get nnz count first
 errval=rsb_mtx_get_coo_block(mtxAp,NULL,NULL,NULL,frA,lrA,fcA,lcA,NULL,NULL,&rnz,flags )
 // allocate VA, IA, JA to rnz elements
 ...
 // get the  rnz  values then
 errval=rsb_mtx_get_coo_block(mtxAp,  VA,  IA,  JA,frA,lrA,fcA,lcA,NULL,NULL,NULL,flags )
-	   \endcode
-
+\endcode
+*/
+/**
 	   \warning Expect this function to change soon (e.g.: have scaling parameters, etc.). Contact the author if you intend to use it.
 	   \see_lib_get
 	 */
@@ -1240,6 +1428,7 @@ rsb_err_t rsb_spmm(rsb_trans_t transA, const void * alphap, const struct rsb_mtx
 
 	   \rsb_transa_mtx_msg
 	   \rsb_num_threads
+	   \rsb_spmm_compact_nrhs_msg
 
 	   \param \rsb_transa_inp_param_msg
 	   \param \rsb_alpha_inp_param_msg
@@ -1252,6 +1441,7 @@ rsb_err_t rsb_spmm(rsb_trans_t transA, const void * alphap, const struct rsb_mtx
 	   \param \rsb_c_inp_param_msg
 	   \param \rsb_ldc_inp_param_msg
  	   \return \rsberrcodemsg
+	   \rsb_librsbpp_env
 	   \see_lib_spmx
 	 */
 	rsb_err_t errval = RSB_ERR_NO_ERROR;
@@ -1286,6 +1476,9 @@ rsb_err_t rsb_spmsp_to_dense(rsb_type_t typecode, rsb_trans_t transA, const void
 
 	   \warning Parameters \c alphap,betap,transA,transB  are not yet taken in consideration. The following defaults are valid: \f$\alpha=1.0\f$ and \f$\beta=1.0\f$, and \c transA=transB=#RSB_TRANSPOSITION_N.
 
+           Example snip:
+	   \snippet examples/snippets.c snip__rsb_spmsp_to_dense
+
 	   \see_lib_gemm
 	 */
 	/* \todo \rsb_todo_unfinished_inc_msg */
@@ -1309,6 +1502,11 @@ rsb_err_t rsb_mtx_rndr(const char * filename, const struct rsb_mtx_t*mtxAp, rsb_
 	   \param \rsb_render_pmheight_inp_param_msg
 	   \param \rsb_render_rflags_inp_param_msg
 
+           Example rendering a sparse matrix to Postscript:
+	   \snippet examples/snippets.c Render a Sparse matrix to Postscript
+
+	   Setting environment variable \c RSB_USE_HOSTNAME=0 prevents hostname being in the EPS plot internal comments.
+
 	   \see_lib_rndr
 	*/
 	rsb_err_t errval = RSB_ERR_NO_ERROR;
@@ -1332,9 +1530,11 @@ rsb_err_t rsb_file_mtx_rndr(void * pmp, const char * filename, rsb_coo_idx_t pml
 	   \param \rsb_render_rflags_inp_param_msg
 	   \return \rsberrcodemsg
 	   
-	   \warning \rsb_warn_not_th_tested_msg
-
 	   \note At the time being, \c pmlWidth is required to be equal to \c pmWidth.
+
+	   Example rendering a matrix from a Matrix Market file to a pixelmap in memory:
+	   \snippet examples/snippets.c Render matrix from Matrix Market pixelmap in memory
+		
 	   \see_lib_rndr
 	*/
 	rsb_err_t errval = RSB_ERR_NO_ERROR;
@@ -1357,6 +1557,10 @@ rsb_err_t rsb_mtx_switch_to_coo(struct rsb_mtx_t * mtxAp, void ** VAp, rsb_coo_i
 
 	   \note \rsb_note_switch_in_place
 	   \warning \rsb_warn_not_th_tested_msg
+
+           Example:
+           \snippet examples/snippets.c snip__rsb_mtx_switch_to_coo
+
 	   \see_lib_conv
 	 */
 	rsb_err_t errval = RSB_ERR_NO_ERROR;
@@ -1375,10 +1579,13 @@ rsb_err_t rsb_mtx_get_prec(void *opdp, const struct rsb_mtx_t * mtxAp, rsb_precf
 	   \param opdp Preconditioner data pointer (output).
 	   \param \rsb_mtxt_inp_param_msg_a
 	   \param prec_flags Valid preconditioner request flags (currently, only #RSB_PRECF_ILU0 is supported; for it, \c *opdp will be overwritten with two \c rsb_mtx_t pointers, respectively a lower and an upper matrix.).
-	   \param ipdp  Preconditioner data pointer (input).
+	   \param ipdp  Preconditioner data pointer (input) (ignored at the moment).
 
 	   \return \rsberrcodemsg
 
+           Example:
+           \snippet examples/snippets.c snip__rsb_mtx_get_prec
+
 	   \note Matrix should be square, have at least two rows, and have at least one nonzero.
 	   \see_lib_get
 	*/
@@ -1405,7 +1612,9 @@ rsb_err_t rsb_mtx_get_info(const struct rsb_mtx_t *mtxAp, enum rsb_mif_t miflags
 
 	   \return \rsberrcodemsg
 
-	   \warning \rsb_warn_not_th_tested_msg
+	   Example snip:
+	   \snippet examples/snippets.c snip__rsb_mtx_get_info
+
 	   \see_lib_info
 	*/
 	rsb_err_t errval = RSB_ERR_UNIMPLEMENTED_YET;
@@ -1428,6 +1637,9 @@ rsb_err_t rsb_mtx_get_info_str(const struct rsb_mtx_t *mtxAp, const rsb_char_t *
 
 	   \return \rsberrcodemsg
 
+	   Example snip:
+	   \snippet examples/snippets.c Get an info string for the matrix
+
 	   \see_lib_info
 	*/
 	/* \warning \rsb_warn_not_th_tested_msg */
@@ -1490,7 +1702,7 @@ rsb_time_t rsb_time(void)
 	   \return A value for the current time, in seconds.
 	   \see_lib_util
 	 */
-	return rsb_do_time();
+	return rsb__do_time();
 }
 
 #if RSB_WANT_COO_BEGIN 
@@ -1514,6 +1726,7 @@ struct rsb_mtx_t * rsb_mtx_alloc_from_coo_begin(rsb_nnz_idx_t nnzA, rsb_type_t t
 	rsb_err_t errval = RSB_ERR_NO_ERROR;
 	struct rsb_mtx_t * mtxAp = NULL;
 	RSB_INTERFACE_PREAMBLE
+	RSB_INITIALIZE_CHECK_MTX_ERRP(errvalp);
 	mtxAp = rsb__do_mtx_alloc_from_coo_begin(nnzA,typecode,nrA,ncA,flagsA,&errval);
 	RSB_INTERFACE_RETURN_MTX_ERRP(mtxAp,errval,errvalp);
 }
@@ -1524,6 +1737,7 @@ rsb_err_t rsb_mtx_alloc_from_coo_end(struct rsb_mtx_t ** mtxApp)
  	   \ingroup rsb_doc_matrix_assembly rsb_doc_rsb
 
 	   Assembles RSB arrays for a matrix in build state created with #rsb_mtx_alloc_from_coo_begin() and populated with #rsb_mtx_set_vals().
+	   \n
 	   After assembly, any operation on the matrix is allowed.
 	  
 	   \param \rsb_mtxt_inp_param_msg_i
@@ -1581,8 +1795,10 @@ rsb_err_t rsb_tune_spmm(struct rsb_mtx_t ** mtxOpp, rsb_real_t *sfp, rsb_int_t *
 	\param \rsb_ldc_inp_param_msg
 	\return \rsberrcodemsg
 
+	 */
+	 /**
 	   Examples:
-	   \code
+\code{.c}
 // obtain best thread count for mtxAp:
 errval = rsb_tune_spmm(NULL  ,&sf,&tn ,maxr,maxt,transA,&alpha,mtxAp,nrhs,order,Bp,ldB,&beta,Cp,ldC);
 
@@ -1606,12 +1822,14 @@ errval = rsb_tune_spmm(&mtxAp,&sf,&tn ,maxr,maxt,transA,&alpha,NULL ,nrhs,order,
 // illegal call:
 assert(mtxOp != NULL && mtxAp != NULL);
 errval = rsb_tune_spmm(&mtxOp,&sf,&tn ,maxr,maxt,transA,&alpha,mtxAp,nrhs,order,Bp,ldB,&beta,Cp,ldC);
-	   \endcode
-
+\endcode
+	 */
+	 /**
 	\warning 
 	\rsb_tune_warning_doc_msg
 	\todo
 	\rsb_tune_todo_doc_msg
+	\rsb_tune__doc_images
 	\see_lib_spmx
 	*/
 	rsb_err_t errval = RSB_ERR_BADARGS;
@@ -1650,6 +1868,7 @@ rsb_err_t rsb_tune_spsm(struct rsb_mtx_t ** mtxOpp, rsb_real_t *sfp, rsb_int_t *
 	\rsb_tune_warning_doc_msg
 	\todo
 	\rsb_tune_todo_doc_msg
+	\rsb_tune__doc_images
 	\see_lib_spsx
 	\see rsb_tune_spmm
 	*/
